@@ -1,21 +1,31 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See the COPYING
  * file for more information.
  */
 
-#include <unistd.h>
-#ifdef WIN32
+#include "common.h"
+
+#if defined(_WIN32) || defined(__OS2__) || defined(__DJGPP__) || defined(_DOS)
 #include <conio.h>
 #endif
-#include <xmp.h>
-#include "common.h"
+
+#if defined(XMP_AMIGA)
+#ifdef __amigaos4__
+#define __USE_INLINE__
+#endif
+#include <proto/exec.h>
+#include <proto/dos.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #ifdef __CYGWIN__
 #include <sys/select.h>
-
 /*
  * from	daniel åkerud <daniel.akerud@gmail.com>
  * date	Tue, Jul 28, 2009 at 9:59 AM
@@ -29,7 +39,7 @@
  * typed out when running xmp. I have not investigated why this is
  * happening, but there is of course a reason why this mode is not
  * enabled by default.
- * 
+ *
  * 2. Do a select() before read()ing if the platform is Cygwin.
  * This makes Cygwin builds work out of the box with no fiddling around,
  * but does impose a neglectible cpu overhead (for Cygwin builds only).
@@ -58,19 +68,14 @@ static int stdin_ready_for_reading(void)
 static int read_key(void)
 {
 	char key;
-	int ret;
+	int ret = 0;
 
-#if defined HAVE_TERMIOS_H && !defined WIN32
-#ifdef __CYGWIN__
-	if (stdin_ready_for_reading())
-#endif
-		ret = read(0, &key, 1);
-#elif defined WIN32
+#if defined(_WIN32) || defined(__OS2__) || defined(__DJGPP__) || defined(_DOS)
 	if (kbhit()) {
 		key = getch();
 		ret = 1;
 	}
-#elif defined __AMIGA__
+#elif defined(XMP_AMIGA)
 	/* Amiga CLI */
 	{
 		BPTR in = Input();
@@ -79,6 +84,11 @@ static int read_key(void)
 			ret = 1;
 		}
 	}
+#elif defined(HAVE_TERMIOS_H)
+	#ifdef __CYGWIN__
+	if (stdin_ready_for_reading())
+	#endif
+	    ret = read(0, &key, 1);
 #else
 	ret = 0;
 #endif
@@ -90,24 +100,16 @@ static int read_key(void)
 	return key;
 }
 
-static void change_sequence(xmp_context handle, struct xmp_module_info *mi, struct control *ctl, int i)
+static void change_sequence(xmp_context handle, const struct xmp_module_info *mi, struct control *ctl, int i)
 {
 	int seq = ctl->sequence;
 
 	seq += i;
 
-	/* This should never happen with libxmp 4.0.5 or newer */
-	while (mi->seq_data[seq].duration <= 0)
-		seq += i;
-
 	if (seq >= mi->num_sequences) {
 		seq = 0;
 	} else if (seq < 0) {
 		seq = mi->num_sequences - 1;
-
-		/* This should never happen with libxmp 4.0.5 or newer */
-		while (mi->seq_data[seq].duration <= 0)
-			seq--;
 	}
 
 	if (seq == ctl->sequence) {
@@ -127,11 +129,11 @@ static void change_sequence(xmp_context handle, struct xmp_module_info *mi, stru
  * ESC [ C - right arrow
  * ESC [ D - left arrow
  */
-void read_command(xmp_context handle, struct xmp_module_info *mi, struct control *ctl)
+void read_command(xmp_context handle, const struct xmp_module_info *mi, struct control *ctl)
 {
 	int cmd;
 
-	cmd = read_key();	
+	cmd = read_key();
 	if (cmd <= 0)
 		return;
 
@@ -185,8 +187,36 @@ void read_command(xmp_context handle, struct xmp_module_info *mi, struct control
 		ctl->loop++;
 		ctl->loop %= 3;
 		break;
+	case 'X':
+		ctl->cur_info = 'X';
+		break;
+	case 'a': {
+		int f;
+
+		ctl->amiga_mixer = !ctl->amiga_mixer;
+
+		/* set player flags */
+		f = xmp_get_player(handle, XMP_PLAYER_FLAGS);
+		if (ctl->amiga_mixer) {
+			xmp_set_player(handle, XMP_PLAYER_FLAGS,
+						f | XMP_FLAGS_A500);
+		} else {
+			xmp_set_player(handle, XMP_PLAYER_FLAGS,
+						f &= ~XMP_FLAGS_A500);
+		}
+
+		/* set current module flags */
+		f = xmp_get_player(handle, XMP_PLAYER_CFLAGS);
+		if (ctl->amiga_mixer) {
+			xmp_set_player(handle, XMP_PLAYER_CFLAGS,
+						f | XMP_FLAGS_A500);
+		} else {
+			xmp_set_player(handle, XMP_PLAYER_CFLAGS,
+						f &= ~XMP_FLAGS_A500);
+		}
+		break; }
 	case 'Z':
-		ctl->cur_seq = 1;
+		ctl->cur_info = 'Z';
 		break;
 	case 'z':
 		ctl->explore ^= 1;
