@@ -2,25 +2,29 @@
 #define XMP_COMMON_H
 
 #ifdef _MSC_VER
-#define PATH_MAX 1024
+#ifndef __cplusplus
 #define inline __inline
-#define open _open
-#define close _close
-#define write _write
-#define lseek _lseek
-#define strdup _strdup
-#define strcasecmp _stricmp
+#endif
 #define snprintf _snprintf
+#define vsnprintf _vsnprintf
 #define kbhit _kbhit
 #define getch _getch
 #endif
 
+#if defined(__MORPHOS__) || defined(__AROS__) || defined(__AMIGA__) \
+ || defined(__amigaos__) || defined(__amigaos4__) || defined(AMIGA)
+#define XMP_AMIGA	1
+#endif
+
 #define NUM_MODES 13
 #define MAX_DRV_PARM 20
 
+
+#include <xmp.h>
+
 struct player_mode {
-	char *name;
-	char *desc;
+	const char *name;
+	const char *desc;
 	int mode;
 };
 
@@ -35,6 +39,7 @@ struct options {
 	int interp;		/* interpolation type */
 	int dsp;		/* dsp effects */
 	int loop;		/* loop module */
+	int loop_time;		/* loop until time */
 	int random;		/* play in random order */
 	int reverse;		/* reverse stereo channels */
 	int vblank;		/* vblank flag */
@@ -52,9 +57,9 @@ struct options {
 	int explore;		/* play all sequences in module */
 	int show_comment;	/* show module comment text */
 	int player_mode;	/* force tracker emulation */
-	int amiga;		/* enable amiga mixer */
-	char *driver_id;	/* sound driver ID */
-	char *out_file;		/* output file name */
+	int amiga_mixer;	/* enable amiga mixer */
+	const char *driver_id;	/* sound driver ID */
+	const char *out_file;	/* output file name */
 	char *ins_path;		/* instrument path */
 	char *driver_parm[MAX_DRV_PARM]; /* driver parameters */
 	char mute[XMP_MAX_CHANNELS];
@@ -69,35 +74,43 @@ struct control {
 	int sequence;		/* Current sequence */
 	int explore;		/* Play all sequences */
 	int cur_info;		/* Display current sequence or mixer type */
-	int amiga;		/* Toggle amiga mixer mode */
+	int amiga_mixer;	/* Toggle amiga mixer mode */
 	int mixer_type;		/* Mixer type (from player) */
 };
 
 extern struct player_mode pmode[];
 
-int report(char *, ...);
+void delay_ms(unsigned int);
+#ifdef XMP_AMIGA
+void amiga_inittimer(void);
+void amiga_getsystime(void *);
+#endif
+
+char *xmp_strdup(const char *);
+int xmp_strcasecmp(const char *, const char *); /* locale-insensitive */
+int report(const char *, ...);
 
 /* option */
 void get_options(int, char **, struct options *);
 int read_config(struct options *);
-void read_modconf(struct options *, unsigned char *);
+void read_modconf(struct options *, const unsigned char *);
 
 /* terminal */
 int set_tty(void);
 int reset_tty(void);
 
 /* info */
-void info_mod(struct xmp_module_info *, int);
-void info_message(char *, ...);
+void info_mod(const struct xmp_module_info *, int);
+void info_message(const char *, ...);
 void info_frame_init(void);
-void info_frame(struct xmp_module_info *, struct xmp_frame_info *, struct control *, int);
-void info_ins_smp(struct xmp_module_info *);
-void info_instruments(struct xmp_module_info *);
-void info_samples(struct xmp_module_info *);
-void info_comment(struct xmp_module_info *);
+void info_frame(const struct xmp_module_info *, const struct xmp_frame_info *, struct control *, int);
+void info_ins_smp(const struct xmp_module_info *);
+void info_instruments(const struct xmp_module_info *);
+void info_samples(const struct xmp_module_info *);
+void info_comment(const struct xmp_module_info *);
 void info_help(void);
 
 /* commands */
-void read_command(xmp_context, struct xmp_module_info *, struct control *);
+void read_command(xmp_context, const struct xmp_module_info *, struct control *);
 
 #endif
