@@ -6,6 +6,8 @@
  * file for more information.
  */
 
+#include "common.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -15,37 +17,38 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#if defined(XMP_AMIGA)
+#include <proto/timer.h>
+#include <time.h>
+#endif
+#if defined(__WATCOMC__)
+#include <time.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <stdarg.h>
-#include <xmp.h>
-#include "errno.h"
-#include "sound.h"
-#include "common.h"
+#if defined(HAVE_GETOPT_H) && defined(HAVE_GETOPT_LONG)
+#include <getopt.h>
+#else
+#include "getopt_long.h"
+#endif
+#include <errno.h>
 
-#ifdef WIN32
+#include "sound.h"
+#include "xmp_version.h"
+
+#ifdef _WIN32
 #include <windows.h>
 #endif
 
-extern int optind;
-
 static struct sound_driver *sound;
 static unsigned int foreground_in, foreground_out;
 static int refresh_status;
 
-
-int report(char *fmt, ...)
-{
-	va_list a;
-	int n;
-
-	va_start(a, fmt);
-	n = vfprintf(stderr, fmt, a);
-	va_end(a);
-
-	return n;
-}
-
 #ifdef HAVE_SIGNAL_H
 static void cleanup(int sig)
 {
@@ -95,6 +98,9 @@ static void sigcont_handler(int sig)
 	}
 #else
 	foreground_in = foreground_out = 1;
+	#if defined(XMP_AMIGA)
+	set_tty();	/* amiga version -- sets stdin to raw mode. */
+	#endif
 #endif
 
 	if (sig != 0)
@@ -105,7 +111,7 @@ static void sigcont_handler(int sig)
 }
 #endif
 
-static void show_info(int what, struct xmp_module_info *mi, int mode)
+static void show_info(int what, const struct xmp_module_info *mi, int mode)
 {
 	report("\r%78.78s\n", " ");
 	switch (what) {
@@ -144,7 +150,7 @@ static void shuffle(int argc, char **argv)
 }
 
 static void check_pause(xmp_context xc, struct control *ctl,
-	struct xmp_module_info *mi, struct xmp_frame_info *fi, int verbose)
+	const struct xmp_module_info *mi, const struct xmp_frame_info *fi, int verbose)
 {
 	if (ctl->pause) {
 		sound->pause();
@@ -152,7 +158,7 @@ static void check_pause(xmp_context xc, struct control *ctl,
 			info_frame(mi, fi, ctl, 1);
 		}
 		while (ctl->pause) {
-			usleep(100000);
+			delay_ms(100);
 			read_command(xc, mi, ctl);
 			if (ctl->display) {
 				if (verbose) {
@@ -169,13 +175,13 @@ static void check_pause(xmp_context xc, struct control *ctl,
 
 static void get_mixer_type(int t, struct options *opt, char *buf, size_t size)
 {
-	char *fmt;
+	const char *fmt;
 
 	fmt = opt->format & XMP_FORMAT_MONO ? "mono" : "stereo";
 
 	switch (t) {
 	case XMP_MIXER_STANDARD: {
-		char *itp;
+		const char *itp;
 		switch (opt->interp) {
 		case XMP_INTERP_NEAREST:
 			itp = "non-interpolated";
@@ -202,9 +208,9 @@ static void get_mixer_type(int t, struct options *opt, char *buf, size_t size)
 	}
 }
 
-static void load_error(char *name, char *filename, int val)
+static void load_error(const char *name, const char *filename, int val)
 {
-	char *msg;
+	const char *msg;
 
 	val = -val;
 	switch (val) {
@@ -242,14 +248,25 @@ int main(int argc, char **argv)
 	int val, lf_flag;
 	int flags;
 	int played;
-#ifndef WIN32
-	struct timeval tv;
-	struct timezone tz;
-
-	gettimeofday(&tv, &tz);
-	srand(tv.tv_usec);
-#else
+#if defined(_WIN32)
+	setvbuf(stderr, NULL, _IONBF, 0);
 	srand(GetTickCount());
+#elif defined(__WATCOMC__)
+	srand(time(NULL));
+#elif defined(__amigaos4__)
+	struct TimeVal tv;
+	amiga_inittimer();
+	amiga_getsystime(&tv);
+	srand(tv.Microseconds);
+#elif defined(XMP_AMIGA)
+	struct timeval tv;
+	amiga_inittimer();
+	amiga_getsystime(&tv);
+	srand(tv.tv_micro);
+#else
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	srand(tv.tv_usec);
 #endif
 
 	init_sound_drivers();
@@ -368,7 +385,7 @@ int main(int argc, char **argv)
 
 	control.loop = opt.loop;
 	control.explore = opt.explore;
-	control.amiga = opt.amiga;
+	control.amiga_mixer = opt.amiga_mixer;
 	first = optind;
 
     play_all:
@@ -398,7 +415,7 @@ int main(int argc, char **argv)
 			load_error(argv[0], argv[optind], val);
 
 			if (skipprev) {
-		        	optind -= 2;
+				optind -= 2;
 				if (optind < (first - 1)) {
 					optind += 2;
 					skipprev = 0;
@@ -483,7 +500,7 @@ int main(int argc, char **argv)
 			set_flag(flags, opt.vblank, XMP_FLAGS_VBLANK);
 			set_flag(flags, opt.fx9bug, XMP_FLAGS_FX9BUG);
 			set_flag(flags, opt.fixloop, XMP_FLAGS_FIXLOOP);
-			set_flag(flags, control.amiga, XMP_FLAGS_A500);
+			set_flag(flags, control.amiga_mixer, XMP_FLAGS_A500);
 			xmp_set_player(xc, XMP_PLAYER_CFLAGS, flags);
 
 			/* Show module data */
@@ -503,7 +520,7 @@ int main(int argc, char **argv)
 					info_comment(&mi);
 				}
 			}
-	
+
 			/* Play sequence */
 
 			refresh_status = 1;
@@ -512,7 +529,7 @@ int main(int argc, char **argv)
     play_sequence:
 			while (!opt.info && xmp_play_frame(xc) == 0) {
 				int old_loop = fi.loop_count;
-				
+
 				xmp_get_frame_info(xc, &fi);
 				control.mixer_type = xmp_get_player(
 						xc, XMP_PLAYER_MIXER_TYPE);
@@ -520,7 +537,9 @@ int main(int argc, char **argv)
 				/* Check loop */
 
 				if (old_loop != fi.loop_count) {
-					if (control.loop == 1) {
+					if (control.loop == 1 ||
+					    (opt.loop_time > 0 &&
+					     control.time < opt.loop_time)) {
 						info_message("Loop sequence %d", control.sequence);
 					} else {
 						break;
@@ -553,11 +572,10 @@ int main(int argc, char **argv)
 						refresh_status = 1;
 					}
 
-					
 					switch (control.cur_info) {
 					case 'X': {
 						char buf[80];
- 						get_mixer_type(control.mixer_type, &opt, buf, 80);
+						get_mixer_type(control.mixer_type, &opt, buf, 80);
 						info_message("Mixer type: %s", buf);
 						control.cur_info = 0;
 						break; }
@@ -628,5 +646,5 @@ int main(int argc, char **argv)
 
 	sound->deinit();
 
-	exit(EXIT_SUCCESS);
+	return 0;
 }
