@@ -10,27 +10,6 @@
 #include <string.h>
 #include "sound.h"
 
-extern struct sound_driver sound_null;
-extern struct sound_driver sound_wav;
-extern struct sound_driver sound_aiff;
-extern struct sound_driver sound_file;
-extern struct sound_driver sound_qnx;
-extern struct sound_driver sound_alsa05;
-extern struct sound_driver sound_oss;
-extern struct sound_driver sound_alsa;
-extern struct sound_driver sound_win32;
-extern struct sound_driver sound_pulseaudio;
-extern struct sound_driver sound_coreaudio;
-extern struct sound_driver sound_hpux;
-extern struct sound_driver sound_sndio;
-extern struct sound_driver sound_sgi;
-extern struct sound_driver sound_solaris;
-extern struct sound_driver sound_netbsd;
-extern struct sound_driver sound_bsd;
-extern struct sound_driver sound_beos;
-extern struct sound_driver sound_amiga;
-extern struct sound_driver sound_aix;
-
 LIST_HEAD(sound_driver_list);
 
 static void register_sound_driver(struct sound_driver *sd)
@@ -70,6 +49,9 @@ void init_sound_drivers(void)
 #ifdef SOUND_COREAUDIO
 	register_sound_driver(&sound_coreaudio);
 #endif
+#ifdef SOUND_OS2DART
+	register_sound_driver(&sound_os2dart);
+#endif
 #ifdef SOUND_WIN32
 	register_sound_driver(&sound_win32);
 #endif
@@ -87,6 +69,9 @@ void init_sound_drivers(void)
 #endif
 #ifdef SOUND_QNX
 	register_sound_driver(&sound_qnx);
+#endif
+#ifdef SOUND_SB
+	register_sound_driver(&sound_sb);
 #endif
 	register_sound_driver(&sound_wav);
 	register_sound_driver(&sound_aiff);
@@ -98,7 +83,7 @@ struct sound_driver *select_sound_driver(struct options *options)
 {
 	struct list_head *head;
 	struct sound_driver *sd;
-	char *pref = options->driver_id;
+	const char *pref = options->driver_id;
 
 	if (pref) {
 		list_for_each(head, &sound_driver_list) {
