@@ -10,14 +10,13 @@
 #include <AudioUnit/AudioUnit.h>
 #include <AudioToolbox/AudioToolbox.h>
 #include <CoreServices/CoreServices.h>
-#include <unistd.h>
 #include "sound.h"
 
 static AudioUnit au;
 
 #ifndef HAVE_AUDIOUNIT_AUDIOCOMPONENT_H
 #define AudioComponent Component
-#define	AudioComponentDescription ComponentDescription
+#define AudioComponentDescription ComponentDescription
 #define AudioComponentFindNext FindNextComponent
 #define AudioComponentInstanceNew OpenAComponent
 #define AudioComponentInstanceDispose CloseComponent
@@ -104,7 +103,7 @@ static int read_buffer(unsigned char *data, int len)
 	return len;
 }
 
-OSStatus render_proc(void *inRefCon,
+static OSStatus render_proc(void *inRefCon,
 		AudioUnitRenderActionFlags *inActionFlags,
 		const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber,
 		UInt32 inNumFrames, AudioBufferList *ioData)
@@ -118,14 +117,13 @@ OSStatus render_proc(void *inRefCon,
 	read_buffer((unsigned char *)ioData->mBuffers[0].mData, amt);
 	ioData->mBuffers[0].mDataByteSize = amt;
 
-        return noErr;
+	return noErr;
 }
 
 /*
  * end of CoreAudio helpers
  */
 
-
 static int init(struct options *options)
 {
 	AudioStreamBasicDescription ad;
@@ -164,7 +162,7 @@ static int init(struct options *options)
 	ad.mBytesPerPacket = ad.mBytesPerFrame;
 	ad.mFramesPerPacket = 1;
 
-        packet_size = ad.mFramesPerPacket * ad.mChannelsPerFrame *
+	packet_size = ad.mFramesPerPacket * ad.mChannelsPerFrame *
 						(ad.mBitsPerChannel / 8);
 
 	cd.componentType = kAudioUnitType_Output;
@@ -192,24 +190,24 @@ static int init(struct options *options)
 		goto err1;
 
 	chunk_size = max_frames;
-        num_chunks = (options->rate * ad.mBytesPerFrame * latency / 1000
+	num_chunks = (options->rate * ad.mBytesPerFrame * latency / 1000
 						+ chunk_size - 1) / chunk_size;
-        buffer_len = (num_chunks + 1) * chunk_size;
+	buffer_len = (num_chunks + 1) * chunk_size;
 	if ((buffer = calloc(num_chunks + 1, chunk_size)) == NULL)
 		goto err;
 
 	rc.inputProc = render_proc;
 	rc.inputProcRefCon = 0;
 
-        buf_read_pos = 0;
-        buf_write_pos = 0;
+	buf_read_pos = 0;
+	buf_write_pos = 0;
 	paused = 1;
 
 	if ((status = AudioUnitSetProperty(au,
 			kAudioUnitProperty_SetRenderCallback,
 			kAudioUnitScope_Input, 0, &rc, sizeof(rc))))
 		goto err2;
-	
+
 	return 0;
 
     err2:
@@ -230,12 +228,12 @@ static void play(void *b, int i)
 
 	/* block until we have enough free space in the buffer */
 	while (buf_free() < i)
-		usleep(100000);
+		delay_ms(100);
 
 	while (i) {
-        	if ((j = write_buffer(b, i)) > 0) {
+		if ((j = write_buffer(b, i)) > 0) {
 			i -= j;
-			b += j;
+			b = (char *)b + j;
 		} else
 			break;
 	}
@@ -249,7 +247,7 @@ static void play(void *b, int i)
 
 static void deinit(void)
 {
-        AudioOutputUnitStop(au);
+	AudioOutputUnitStop(au);
 	AudioUnitUninitialize(au);
 	AudioComponentInstanceDispose(au);
 	free(buffer);
@@ -261,7 +259,7 @@ static void flush(void)
 
 static void onpause(void)
 {
-        AudioOutputUnitStop(au);
+	AudioOutputUnitStop(au);
 }
 
 static void onresume(void)
