@@ -7,27 +7,16 @@
  */
 
 #include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <string.h>
-#include <unistd.h>
 #include "sound.h"
 
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
-static int fd;
+static FILE *fd;
 static long size;
 static int swap_endian;
 
-struct sound_driver sound_file;
-
 static int init(struct options *options)
 {
 	char **parm = options->driver_parm;
-	char *buf;
 
 	swap_endian = 0;
 
@@ -41,24 +30,11 @@ static int init(struct options *options)
 	}
 
 	if (strcmp(options->out_file, "-")) {
-		fd = open(options->out_file, O_WRONLY | O_CREAT | O_TRUNC
-							| O_BINARY, 0644);
-		if (fd < 0)
+		fd = fopen(options->out_file, "wb");
+		if (fd == NULL)
 			return -1;
 	} else {
-		fd = 1;
-	}
-
-	if (strcmp(options->out_file, "-")) {
-		int len = strlen(sound_file.description) +
-				strlen(options->out_file) + 8;
-		if ((buf = malloc(len)) == NULL)
-			return -1;
-		snprintf(buf, len, "%s: %s", sound_file.description,
-						options->out_file);
-		sound_file.description = buf;
-	} else {
-		sound_file.description = strdup("stdout");
+		fd = stdout;
 	}
 
 	return 0;
@@ -67,19 +43,25 @@ static int init(struct options *options)
 static void play(void *b, int len)
 {
 	if (swap_endian) {
-		convert_endian(b, len);
+		convert_endian((unsigned char *)b, len);
 	}
-	write(fd, b, len);
+	fwrite(b, 1, len, fd);
 	size += len;
 }
 
 static void deinit(void)
 {
-	free(sound_file.description);
+	if (fd && fd != stdout) {
+		fclose(fd);
+	}
+	fd = NULL;
 }
 
 static void flush(void)
 {
+	if (fd) {
+		fflush(fd);
+	}
 }
 
 static void onpause(void)
@@ -90,7 +72,7 @@ static void onresume(void)
 {
 }
 
-static const char *const help[] = {
+static const char * const help[] = {
 	"endian=big", "Force big-endian 16-bit samples",
 	"endian=little", "Force little-endian 16-bit samples",
 	NULL
@@ -107,4 +89,3 @@ struct sound_driver sound_file = {
 	onpause,
 	onresume
 };
-
